Subject: Re: bug#25086: [PATCH] Fix the timezone detection of parse-iso8601-time-string.

From: Hong Xu <[email-masked]>
Date: Sat, 17 Dec 2016 12:42:57 -0800

On 2016-12-17 Sat 06:58 GMT-0800, Eli Zaretskii <[email-masked]> wrote:

>> From: Hong Xu <[email-masked]>
>> Date: Thu, 01 Dec 2016 20:04:00 -0800
>>
>> +;;;###autoload
>>  (defun parse-iso8601-time-string (date-string)
>> +  "Parse an ISO 8601 time string, such as 2016-12-01T23:35:06-05:00.
>> +If DATE-STRING cannot be parsed, it falls back to
>> +`parse-time-string'."
>
> Why did you want to autoload this?  The log message doesn't mention
> this change at all.

The attached new version now fixes this issue in the log message. It
autoloads because parse-time-string autoloads -- If parse-time-string is
useful for autoloading, there is no reason that
parse-iso8601-time-string is not useful for that.

>
>> +                 '(42 35 19 22 2 2016 1 t -25200)))
>> +  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54-0200")
>> +                 '(13818 33666)))
>> +  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54-02:00")
>> +                 '(13818 33666)))
>> +  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54-02")
>> +                 '(13818 33666)))
>> +  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54+02")
>> +                 '(13818 19266))))
>
> Does this also test this part of your changes:
>
>> @@ -204,13 +204,17 @@ parse-time-iso8601-regexp
>>  	 (date-time (concat full-date "T" full-time)))
>>      (list (concat "^" full-date)
>>  	  (concat "T" partial-time)
>> -	  (concat "Z" time-numoffset)))
>> +	  (concat "Z?" time-numoffset)))
>
> If not, could you please add tests for that?
>

I've added the tests now. And indeed, it exposes a bug in the original patch.

Again, I decomposed the patch to two different parts, one for the stable
branch and the other (with only test files) for the master branch.

From 39e4d94da9b8806871cb2c9145cd1f14233526f5 Mon Sep 17 00:00:00 2001
From: Hong Xu <[email-masked]>
Date: Sat, 17 Dec 2016 12:07:45 -0800
Subject: [PATCH] Fix the timezone detection of parse-iso8601-time-string.

        * parse-time.el (parse-iso8601-time-string): Fix its timezone
        parsing and make it autoload.
        * parse-time.el: Add doc for parse-iso8601-time-string and
        make it autoload.
        * editfns.c (Fdecode-time): Minor doc improvement for decode-time.
        * emacs-mime.texi (time-date): Add an example for
        parse-iso8601-time-string.
---
 doc/misc/emacs-mime.texi    |  3 +++
 lisp/calendar/parse-time.el | 19 ++++++++++++-------
 src/editfns.c               |  5 ++---
 3 files changed, 17 insertions(+), 10 deletions(-)

diff --git a/doc/misc/emacs-mime.texi b/doc/misc/emacs-mime.texi
index 4d68246bba4b..8d6536265416 100644
--- a/doc/misc/emacs-mime.texi
+++ b/doc/misc/emacs-mime.texi
@@ -1536,6 +1536,9 @@ time-date
 (date-to-time "Sat Sep 12 12:21:54 1998 +0200")
 @result{} (13818 19266)
 
+(parse-iso8601-time-string "1998-09-12T12:21:54+0200")
+@result{} (13818 19266)
+
 (float-time '(13818 19266))
 @result{} 905595714.0
 
diff --git a/src/editfns.c b/src/editfns.c
index 5cc4a67ab19b..2f1f412d9e4e 100644
--- a/src/editfns.c
+++ b/src/editfns.c
@@ -2101,9 +2101,8 @@ format_time_string (char const *format, ptrdiff_t formatlen,
 
 DEFUN ("decode-time", Fdecode_time, Sdecode_time, 0, 2, 0,
        doc: /* Decode a time value as (SEC MINUTE HOUR DAY MONTH YEAR DOW DST UTCOFF).
-The optional SPECIFIED-TIME should be a list of (HIGH LOW . IGNORED),
-as from `current-time' and `file-attributes', or nil to use the
-current time.
+The optional TIME should be a list of (HIGH LOW . IGNORED), as from
+`current-time' and `file-attributes', or nil to use the current time.
 It can also be a single integer number of seconds since the epoch.
 The obsolete form (HIGH . LOW) is also still accepted.
 The optional ZONE is omitted or nil for Emacs local time, t for
-- 
2.1.4

From 9f06f66c4d5970c4cf692956544456ab4458f3b1 Mon Sep 17 00:00:00 2001
From: Hong Xu <[email-masked]>
Date: Sat, 17 Dec 2016 12:38:26 -0800
Subject: [PATCH] Add tests for parse-iso8601-time-string.

	* parse-time-tests.el (parse-time-tests): Add tests for
	parse-iso8601-time-string.
---
 test/lisp/calendar/parse-time-tests.el | 14 +++++++++++++-
 1 file changed, 13 insertions(+), 1 deletion(-)

diff --git a/test/lisp/calendar/parse-time-tests.el b/test/lisp/calendar/parse-time-tests.el
index 9bcf2b4a53c7..7a087d0f0bee 100644
--- a/test/lisp/calendar/parse-time-tests.el
+++ b/test/lisp/calendar/parse-time-tests.el
@@ -42,7 +42,19 @@
   (should (equal (parse-time-string "Monday, 22 february 2016 19:35:42 +0100")
                  '(42 35 19 22 2 2016 1 nil 3600)))
   (should (equal (parse-time-string "Monday, 22 february 2016 19:35:42 PDT")
-                 '(42 35 19 22 2 2016 1 t -25200))))
+                 '(42 35 19 22 2 2016 1 t -25200)))
+  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54-0200")
+                 '(13818 33666)))
+  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54-02:00")
+                 '(13818 33666)))
+  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54-02")
+                 '(13818 33666)))
+  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54+02")
+                 '(13818 19266)))
+  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54Z")
+                 '(13818 26466)))
+  (should (equal (parse-iso8601-time-string "1998-09-12T12:21:54")
+                 '(13818 26466))))
 
 (provide 'parse-time-tests)
 
diff --git a/lisp/calendar/parse-time.el b/lisp/calendar/parse-time.el
index 6ba26a4a00d0..6bd748cbe0d9 100644
--- a/lisp/calendar/parse-time.el
+++ b/lisp/calendar/parse-time.el
@@ -195,7 +195,7 @@ parse-time-iso8601-regexp
 	 (time-minute 2digit)
 	 (time-second 2digit)
 	 (time-secfrac "\\(\\.[0-9]+\\)?")
-	 (time-numoffset (concat "[-+]\\(" time-hour "\\):" time-minute))
+	 (time-numoffset (concat "\\([-+]" time-hour "\\):?" time-minute "?"))
 	 (time-offset (concat "Z" time-numoffset))
 	 (partial-time (concat time-hour colon time-minute colon time-second
 			       time-secfrac))
@@ -204,19 +204,24 @@ parse-time-iso8601-regexp
 	 (date-time (concat full-date "T" full-time)))
     (list (concat "^" full-date)
 	  (concat "T" partial-time)
-	  (concat "Z" time-numoffset)))
+	  (concat "Z?" time-numoffset)))
   "List of regular expressions matching ISO 8601 dates.
 1st regular expression matches the date.
 2nd regular expression matches the time.
 3rd regular expression matches the (optional) timezone specification.")
 
+;;;###autoload
 (defun parse-iso8601-time-string (date-string)
+  "Parse an ISO 8601 time string, such as 2016-12-01T23:35:06-05:00.
+If DATE-STRING cannot be parsed, it falls back to
+`parse-time-string'."
   (let* ((date-re (nth 0 parse-time-iso8601-regexp))
 	 (time-re (nth 1 parse-time-iso8601-regexp))
 	 (tz-re (nth 2 parse-time-iso8601-regexp))
-	 re-start
-	 time seconds minute hour fractional-seconds
-	 day month year day-of-week dst tz)
+         (tz 0)
+         re-start
+         time seconds minute hour fractional-seconds
+         day month year day-of-week dst)
     ;; We need to populate 'time' with
     ;; (SEC MIN HOUR DAY MON YEAR DOW DST TZ)
 
@@ -235,10 +240,10 @@ parse-iso8601-time-string
                                                     "0"))
 	      re-start (match-end 0))
 	(when (string-match tz-re date-string re-start)
-	  (setq tz (match-string 1 date-string)))
+          (setq tz (* 3600 (string-to-number (match-string 1 date-string)))))
 	(setq time (list seconds minute hour day month year day-of-week dst tz))))
 
     ;; Fall back to having Gnus do fancy things for us.
     (when (not time)
       (setq time (parse-time-string date-string)))
 
@@ -235,10 +240,10 @@ parse-iso8601-time-string
 	(setq time (list seconds minute hour day month year day-of-week dst tz))))
 
-    ;; Fall back to having Gnus do fancy things for us.
+    ;; Fall back to having `parse-time-string' do fancy things for us.
     (when (not time)
       (setq time (parse-time-string date-string)))
-- 
2.1.4